<template>
	<div class="flex h-full">
		<div class="flex flex-col h-full flex-auto">
			<div class="h-full flex flex-col items-center overflow-y-auto">
				<div ref="chatListDom" class="h-full" :style="{ width: chatWidth }">
					<div
						class="group flex px-4 py-6 hover:bg-slate-100 rounded-lg relative"
						:class="{ 'flex-row-reverse': item.role === RoleEnum.user }"
						v-for="(item, index) of computedMessageList"
						:key="index"
					>
						<img
							class="rounded-full size-12 flex-0"
							:class="{ 'mr-4': item.role === RoleEnum.assistant, 'ml-4': item.role === RoleEnum.user }"
							:src="roleImageMap[item.role]"
							alt=""
							srcset=""
						/>
						<div class="flex-auto flex" :class="{ 'justify-end': item.role === RoleEnum.user }">
							<div class="inline-flex flex-col" :class="{ 'w-full': item.role === RoleEnum.assistant }">
								<div class="w-full" v-if="item.content?.values">
									<div
										class="text-sm rounded-[6px] p-4 leading-relaxed"
										:style="{ backgroundColor: item.role === RoleEnum.user ? 'rgb(197 224 255)' : 'white' }"
									>
										<div v-if="item.content.errCode === ErrorCode.Message" class="flex-column w-full">
											<p class="text-red-500">
												{{ item.content.errMsg }}
											</p>
											<div class="mt-5 flex items-center" v-if="showFixQuestion(item)">
												<div class="text-gray-600 flex-0">
													{{ item.content.origin.err_json.fix_question.title + '：' }}
												</div>
												<div class="ml-1 space-x-2 inline-flex flex-wrap">
													<div
														v-for="fixItem in item.content.origin.err_json.fix_question?.values"
														:key="fixItem"
														class="bg-gray-200 p-3 hover:bg-[#c5e0ff] hover:text-[#1c86ff] cursor-pointer rounded-lg"
														@click="fixQuestionClick(fixItem, item.content.origin)"
													>
														{{ fixItem.title }}
													</div>
												</div>
											</div>
										</div>
										<template v-else>
											<component :is="answerTypeMapCom[item.content.type]" :data="item.content.values" :originData="item" />

											<div
												v-if="item.role === RoleEnum.assistant && item.content.origin?.ext_call_list"
												class="flex font-bold items-center mt-6"
											>
												<div class="flex-0 mb-auto -mr-4">关联功能：</div>
												<div class="space-x-5 flex flex-wrap">
													<div
														v-for="callItem in item.content.origin?.ext_call_list"
														:key="callItem.call_ext_id"
														@click="relativeQueryClick(callItem)"
														class="cursor-pointer hover:underline first-of-type:ml-5"
													>
														{{ callItem.question }}
													</div>
												</div>
											</div>
										</template>
									</div>

									<!-- 操作 -->
									<div v-if="item.role === RoleEnum.assistant" class="absolute flex items-center right-0 mr-4 mt-2 space-x-2">
										<div
											class="flex items-center justify-center size-[15px]"
											v-if="item.content?.type === AnswerType.Text || item.content?.type === AnswerType.Knowledge"
										>
											<i
												class="p-2 ywifont ywicon-copy cursor-pointer hover:text-[#0284ff] hover:!text-[18px]"
												@click="copyClick(item)"
											/>
										</div>
										<template v-if="item.content.errCode !== ErrorCode.Message">
											<div class="flex items-center justify-center size-[15px]">
												<i
													:class="{ 'text-[#0284ff]': item.state === AnswerState.Like }"
													class="p-2 ywifont ywicon-dianzan cursor-pointer hover:text-[#0284ff] font-medium hover:!text-[18px]"
													@click="likeClick(item)"
												/>
											</div>
											<div class="flex items-center justify-center size-[15px]">
												<i
													:class="{ 'text-[#0284ff]': item.state === AnswerState.Unlike }"
													class="p-2 ywifont ywicon-buzan cursor-pointer hover:text-[#0284ff] !text-[13px] hover:!text-[15px]"
													@click="unLikeClick(item)"
												/>
											</div>
										</template>

										<div class="flex items-center justify-center size-[15px] relative">
											<i
												class="p-2 ywifont ywicon-wentifankui cursor-pointer hover:text-[#0284ff] !text-[13px] hover:!text-[15px]"
												@click="
													($event) =>
														feedbackClick(
															$event,
															item,
															computedMessageList
																.filter((v) => v.role === RoleEnum.assistant)
																.findIndex((v) => v.historyId === item.historyId)
														)
												"
											/>
											<FeedbackPanel
												v-show="feedbackIsShow && currentFeedbackMapItem === item"
												ref="feedbackPanelRef"
												v-model:isShow="feedbackIsShow"
												v-model:content="feedbackContent"
												:chatItem="currentFeedbackMapItem"
												:position="feedbackPosition"
											/>
										</div>
									</div>
								</div>

								<Loding v-if="isTalking && index === messageList.length - 1" class="w-fit" :process="process" />
							</div>
						</div>
					</div>
					<div v-if="showAskMore" class="ml-4 mt-5 text-sm">
						<div class="text-gray-600 mb-5">你可以继续问我：</div>
						<div class="space-y-2 inline-flex flex-col">
							<div
								v-for="item in computedMessageList.at(-1).content.askMoreList"
								:key="item.history_id"
								class="bg-white p-3 hover:bg-[#c5e0ff] hover:text-[#1c86ff] cursor-pointer rounded-lg"
								@click="askMoreClick(item)"
							>
								{{ item.question }}
							</div>
						</div>
					</div>
				</div>
			</div>

			<div class="sticky bottom-0 w-full p-6 pb-8 bg-[rgb(247,248,250)] flex justify-center">
				<PlayBar
					v-model:voicePageIsShow="voicePageIsShow"
					:isTalking="isTalking"
					:isHome="false"
					v-model="messageContent.values"
					@sendClick="sendClick"
					:style="{ width: chatWidth }"
				></PlayBar>
			</div>
		</div>

		<CustomDrawer v-model:isShow="drawerIsShow" @updateChatInput="updateChatInput" />
	</div>
</template>

<script setup lang="ts">
import { ElMessage } from 'element-plus';
import _ from 'lodash';
import { v4 as uuidv4 } from 'uuid';
import { computed, onMounted, ref } from 'vue';
import FeedbackPanel from './components/FeedbackPanel.vue';
import Loding from './components/Loding.vue';
import { useAssistantContentOpt } from './hooks/useAssistantContentOpt';
import { useQueryProcess } from './hooks/useQueryProcess';
import { useScrollToBottom } from './hooks/useScrollToBottom';
import type { ChatContent } from './model/types';
import { AnswerState, AnswerType, RoleEnum, answerTypeMapCom, roleImageMap, type ChatMessage } from './model/types';
import { GetHistoryAnswer, QueryHistoryDetail, QuestionAi, extCallQuery } from '/@/api/ai/chat';
import PlayBar from '/@/components/chat/components/playBar/PlayBar.vue';
import CustomDrawer from '/@/components/drawer/CustomDrawer.vue';
import router from '/@/router';
import { activeChatRoom, activeLLMId, activeSampleId, activeSectionAId, getRoomConfig, roomConfig } from '/@/stores/chatRoom';
import { ErrorCode } from '/@/utils/request';

const chatWidth = '75%';
const voicePageIsShow = ref(false);
let isTalking = ref(false);
let messageContent = ref<ChatContent>({
	type: AnswerType.Text,
	values: '',
});
const currentRoute = router.currentRoute;
const currentRouteId = currentRoute.value.query.id as string;
const chatListDom = ref<HTMLDivElement>();
const messageList = ref<ChatMessage[]>([]);
const computedMessageList = computed(() => {
	return messageList.value.filter((v) => !!v);
});

const parseContent = (res) => {
	if (!res) return null;
	let content: ChatContent = {
		type: AnswerType.Text,
		values: '解析失败！',
	};

	switch (res.answer_type) {
		case AnswerType.RecordSet:
			content = {
				type: AnswerType.RecordSet,
				values: res.values,
			};
			break;
		case AnswerType.Text:
			content = {
				type: AnswerType.Text,
				values: res.values ?? res.answer,
			};
			break;

		case AnswerType.Knowledge:
			content = {
				type: AnswerType.Knowledge,
				values: res.knowledge,
			};

			break;
		case AnswerType.Summary:
			content = {
				type: AnswerType.Summary,
				values: res.summary,
			};
			break;
		case AnswerType.Url:
			content = {
				type: AnswerType.Url,
				values: res.url,
			};
			break;
		case AnswerType.Map:
			content = {
				type: AnswerType.Map,
				values: res.values,
			};
			break;
		default:
			content = {
				type: AnswerType.Text,
				values: '解析失败！',
			};
			break;
	}
	content.askMoreList = _.orderBy(res.context_history, [(item) => Number(item.radio)], ['desc']);
	content.errCode = res?.err_code;
	content.errMsg = res?.json_msg;
	content.origin = res;
	return content;
};

const { clearQueryProcess, process, processId, queryProcess } = useQueryProcess();

let questionRes = null;
const questionAi = async (text) => {
	if (!currentSectionId) {
		ElMessage.warning('发送失败，未确定应用场景！');
	}
	processId.value = uuidv4();
	let judgeParams = null;
	if (!preQuestion.value) {
		// const aiContent = computedMessageList.value.filter((item) => item.role === RoleEnum.assistant);
		// const lastQuestion = aiContent[aiContent.length - 2]?.content?.origin?.question;
		// judgeParams = lastQuestion
		// 	? {
		// 			prev_question: lastQuestion,
		// 	  }
		// 	: {};
			// 正常回答暂时不采用
		judgeParams = {};
	} else {
		judgeParams = {
			prev_question: preQuestion.value,
		};
	}
	
	const params = {
		process_id: processId.value,
		question: text,
		// FIXME: 暂时这样
		section_a_id: currentSectionId,
		history_group_id: currentRouteId,
		raw_mode: roomConfig.value?.[currentRouteId]?.isAnswerByLLM ?? false,
		...judgeParams,
	} as any;

	if (currentSampleId) {
		params.sample_id = currentSampleId;
	}

	if (currentLLMId) {
		params.llm_id = currentLLMId;
	}
	clearQueryProcess();
	queryProcess();
	const res = await QuestionAi(params).finally(() => {
		clearQueryProcess();
	});
	questionRes = res;
	const content = parseContent(res);
	return content;
};

const clearMessageContent = () =>
	(messageContent.value = {
		type: AnswerType.Text,
		values: '',
	});

let currentSectionId = null;
let currentSampleId = null;

let currentLLMId = null;

const getAnswerById = async (historyId: string) => {
	return await GetHistoryAnswer({
		history_id: historyId,
	});
};

const sendChatMessage = async (content: ChatContent = messageContent.value, cb?: any, isCallExtParams?: any) => {
	if (!content?.values || isTalking.value) return;
	const isNewChat = messageList.value.length === 0;
	if (isNewChat) {
		if (activeSampleId.value) {
			currentSampleId = activeSampleId.value;
		}

		if (activeLLMId.value) {
			currentLLMId = activeLLMId.value;
		}

		if (activeSectionAId.value) {
			currentSectionId = activeSectionAId.value;
		}
	}
	let resMsgContent: ChatContent = null;

	try {
		isTalking.value = true;
		const userItem: ChatMessage = { role: RoleEnum.user, content } as any;
		const assistantItem: ChatMessage = { role: RoleEnum.assistant, content: null, state: AnswerState.Null } as any;
		// 发送当前
		messageList.value.push(userItem);
		// 清空输入框
		clearMessageContent();

		// 出现回复，置空出现等待动画
		messageList.value.push(assistantItem);
		if (isCallExtParams) {
			const extRes = await extCallQuery(isCallExtParams);
			questionRes = extRes;
			resMsgContent = parseContent(extRes);
		} else {
			resMsgContent = await questionAi(content.values);
		}

		if (isNewChat) {
			const firstResCb = getRoomConfig(currentRouteId, 'firstResCb');
			firstResCb?.(resMsgContent);
		} else {
			cb?.(resMsgContent);
		}
		userItem.historyId = questionRes.history_id;
		userItem.content.values = questionRes?.question ?? userItem.content.values;
		assistantItem.historyId = questionRes.history_id;
		appendLastMessageContent(resMsgContent);
	} catch (error: any) {
		// appendLastMessageContent({
		// 	type: AnswerType.Text,
		// 	values: '发生错误！',
		// });
	} finally {
		isTalking.value = false;
	}
};

const sendClick = (cb) => {
	sendChatMessage(messageContent.value, cb);
};
const appendLastMessageContent = (content: ChatContent) => {
	if (messageList.value.at(-1)) {
		messageList.value.at(-1).content = content;
	}
};

onMounted(async () => {
	const res = await QueryHistoryDetail({
		history_group_id: currentRouteId,
	});

	messageList.value = (res.details ?? []).map((item) => {
		return {
			historyId: item.history_id,
			role: RoleEnum.user,
			content: {
				type: AnswerType.Text,
				values: item.question,
			},
		} as ChatMessage;
	});
	currentSectionId = res?.details?.[0]?.section_a_id;
	currentSampleId = res?.details?.[0]?.sample_id;
	const resList = await Promise.all((res.details ?? []).map((item) => getAnswerById(item.history_id)));
	let i = 0;
	resList.map((item, index) => {
		const insertIndex = index + 1 + i;
		const userMsg = messageList.value[insertIndex - 1];
		userMsg.content.values = item?.answer?.question ??userMsg.content.values;
		messageList.value.splice(
			insertIndex,
			0,
			item.answer === null
				? null
				: {
						historyId: item.answer?.history_id,
						role: RoleEnum.assistant,
						content: parseContent(item.answer),
						state: item.answer_state,
				  }
		);
		i++;
	});

	if (messageList.value.length === 0) {
		messageContent.value = {
			type: AnswerType.Text,
			values: activeChatRoom.value.title,
		};

		sendChatMessage();
	}
});

const { forbidScroll } = useScrollToBottom({
	chatListDom: chatListDom,
	displayMessageList: computedMessageList,
});

//#region ====================== 关联查询 ======================
const relativeQueryClick = async (val) => {
	sendChatMessage(
		{
			type: AnswerType.Text,
			values: val.question,
		},
		undefined,
		{
			history_group_id: currentRouteId,
			question: val.question,
			call_ext_id: val.call_ext_id,
			call_ext_args: val.agrs ? JSON.stringify(val.agrs) : null,
		}
	);
};
//#endregion

const {
	copyClick,
	likeClick,
	unLikeClick,
	feedbackPosition,
	feedbackIsShow,
	feedbackContent,
	feedbackPanelRef,
	currentFeedbackMapItem,
	feedbackClick,
	askMoreClick,
	fixQuestionClick,
	preQuestion,
	showFixQuestion,
	showAskMore,
} = useAssistantContentOpt({
	forbidScroll,
	sendChatMessage,
	displayMessageList: computedMessageList,
});

//#region ====================== 侧边栏drawer ======================
const drawerIsShow = ref(false);

const updateChatInput = (content) => {
	messageContent.value.values = content;
};
//#endregion
</script>

<style scoped>
pre {
	font-family: -apple-system, 'Noto Sans', 'Helvetica Neue', Helvetica, 'Nimbus Sans L', Arial, 'Liberation Sans', 'PingFang SC',
		'Hiragino Sans GB', 'Noto Sans CJK SC', 'Source Han Sans SC', 'Source Han Sans CN', 'Microsoft YaHei', 'Wenquanyi Micro Hei',
		'WenQuanYi Zen Hei', 'ST Heiti', SimHei, 'WenQuanYi Zen Hei Sharp', sans-serif;
}
</style>
